<?php
/*
Plugin Name: Developer Formatter
Plugin URI: http://wordpress.org/extend/plugins/devformatter/
Description: Developer Formatter system to WordPress. Powered by <a href="http://qbnz.com/highlighter/">GeSHi</a> server-side highlight system.
Version: 2009.0.1.21
Author: Gilberto Saraiva
Author URI: http://gsaraiva.projects.pro.br/

Copyright (c) 2009 Gilberto Saraiva
Released under the GNU General Public License (GPL)
http://www.gnu.org/licenses/gpl.txt
*/

  define('DEVFMT_FOLDER', dirname(plugin_basename(__FILE__)), TRUE);
  define('DEVFMT_PATH', dirname(__FILE__).DIRECTORY_SEPARATOR, TRUE);
  define('DEVFMT_URL', get_bloginfo('wpurl')."/wp-content/plugins/devformatter/");
  define('DEVFMT_GESHIPATH', DEVFMT_PATH."geshi".DIRECTORY_SEPARATOR."geshi");

  $DevFmt_FiltersToAvoid = Array("wptexturize", "wpautop", "force_balance_tags");
  $DefaultFilters = Array();
  $DevFmt_CodeIndex = 0;

  require_once(DEVFMT_PATH.'devcommon.php');
  require_once(DEVFMT_PATH.'devinterface.php');
  require_once(DEVFMT_PATH.'devinstall.php');

  function devfmt_GeSHiParseCode($ACode, $ALang, $ADevFmt = false){
    GLOBAL $DevFmt_Config;
    $geshi = new GeSHi(chr(2).$ACode.chr(2), $ALang, $DevFmt_Config['geshilangpath']);
    $geshi->set_overall_style("");
    $geshi->set_overall_class("");
    $geshi->set_overall_id("");
    $geshi->enable_classes(false);
    $geshi->set_code_style("");
    $geshi->enable_keyword_links($DevFmt_Config['geshiuselink']);
    $geshi->enable_line_numbers(false);
    $geshi->set_header_type(GESHI_HEADER_PRE);
    $code = $geshi->parse_code();
    $code = substr($code, strpos($code, ">") + 1);
    $code = substr($code, 0, strrpos($code, "</") - 1);
    return str_replace(chr(2), "", $code);
  }

  function devfmt_ParseCode($ACode, $ALang, $ADevFmt){
    if($ACode == "") return $ACode;
    $code = $ACode;
    $code = str_replace("\r\n", "\n", $code);
    $code = str_replace("\n", "\r\n", $code);

    $OtherLang = true;
    $WebServerSideLangs = Array(
      "php"       => Array("#<\?(.*?)\?>#s", "<\?(.*?)\?>", "<?", "?>"),
      "php-brief" => Array("#<\?(.*?)\?>#s", "<\?(.*?)\?>", "<?", "?>"),
      "asp"       => Array("#<\%(.*?)\%>#s", "<\%(.*?)\%>", "<%", "%>")
    );

    foreach($WebServerSideLangs as $Lang => $Checkers){
      if(($ALang == $Lang) && (preg_match($Checkers[0], $code, $matches))){
        $OtherLang = false;
        $SvLang = array(array(uniqid(''), $Checkers[1]));

        while(preg_match($Checkers[0], $code, $match)){
          $check = $match[0];
          array_shift($match);
          $SvLang[] = $match;
          $code = str_replace($check, $SvLang[0][0].(Count($SvLang) - 1)." ", $code);
        }

        $html = Array();
        for($i = 1; $i < Count($SvLang); $i++){
          $html[] = Array($SvLang[0][0].($i - 1)." ", $SvLang[0][0].$i." ");
        }

        $code = devfmt_ParseCode($code, "html", $ADevFmt);

        for($i = 1; $i < Count($SvLang); $i++){
          $SvLang[$i][0] = devfmt_GeshiParseCode($Checkers[2] . $SvLang[$i][0] . $Checkers[3], $ALang, $ADevFmt);

          $OpenTags = Array();
          if(preg_match("/\n(.*?)".$SvLang[0][0].$i." /", $code, $match)){
            $tmp = $match[1];
            while(preg_match("/(<(\w+)([^>]*)>(.*?)<\/)/", $tmp, $match))
              $tmp = preg_replace("/(<(".$match[2].")([^>]*)>([^<]*)<\/".$match[2].">)/", ".", $tmp, 1);
            preg_match_all("/(<(\w+)([^>]*)>)/", $tmp, $matches);
            $OpenTags = $matches[0];
          }

          if(Count($OpenTags) > 0){
            $SvLang[$i][0] = str_repeat("</span>", Count($OpenTags)).
              $SvLang[$i][0].implode("", $OpenTags);
          }

          $code = preg_replace("#".$SvLang[0][0].$i." #", $SvLang[$i][0], $code, 1);
        }

        break;
      }
    }

    // Handle HTML format [HTML, JavaScript, Style]
    if($ALang == "html"){
      $OtherLang = false;
      $SvScript = Array();
      while(preg_match("#<script(.*?)>(.*?)</script>#s", $code, $matches)){
        $id = $unid."script".Count($SvScript);
        if($matches[2] != ""){
          $subcode = devfmt_GeshiParseCode($matches[2], "javascript", $ADevFmt);
        }else
          $subcode = "";

        $SvScript[] = Array($id, $matches[1], $subcode);
        $code = str_replace($matches[0], $id, $code);
      }

      $SvStyle = Array();
      while(preg_match("#<style(.*?)>(.*?)</style>#s", $code, $matches)){
        $id = $unid."style".Count($SvStyle);
        if($matches[2] != ""){
          $subcode = devfmt_GeshiParseCode($matches[2], "css", $ADevFmt);
        }else
          $subcode = "";

        $SvStyle[] = Array($id, $matches[1], $subcode);
        $code = str_replace($matches[0], $id, $code);
      }

      foreach($SvScript as $script)
        $code = str_replace($script[0], "<script".$script[1].">".$script[0]."</script>", $code);

      foreach($SvStyle as $style)
        $code = str_replace($style[0], "<style".$style[1].">".$style[0]."</style>", $code);

      $code = devfmt_GeshiParseCode($code, $ALang, $ADevFmt);

      foreach($SvScript as $script)
        $code = str_replace($script[0], $script[2], $code);

      foreach($SvStyle as $style)
        $code = str_replace($style[0], $style[2], $code);
    }

    // Handle others formats
    if($OtherLang)
      $code = devfmt_GeshiParseCode($code, $ALang, $ADevFmt);

    return $code;
  }

  function devfmt_ParseStructure($ACode, $ALang, $AOptions, $AStyle = ""){
    GLOBAL $DevFmt_Config, $DevFmt_CodeIndex;    
    if($AStyle != "") $AStyle = " style=\"".$AStyle."\"";
    
    $NewCode = "<table class=\"devcodearea\" width=\"100%\">";
    $CodeLines = explode("\n", $ACode);
    $LastOpenTags = "";
    $TrOdd = false;
    $PadSize = strlen("" . Count($CodeLines));
    for($i = 0; $i < Count($CodeLines); $i++){
      $CodeLines[$i] = $LastOpenTags . $CodeLines[$i];
      $tmp = $CodeLines[$i];
      if(preg_match_all("/(<(\w+)([^>]*)>(.*?)<\/)/", $tmp, $matchsall))
        foreach($matchsall[2] as $matchs)
          $tmp = preg_replace("/(<(".$matchs.")([^>]*)>([^<]*)<\/".$matchs.">)/", "", $tmp);

      $LastOpenTags = "";
      if(preg_match_all("/(<(\w+)([^>]*)>)/", $tmp, $matchs))
        $LastOpenTags = implode("", $matchs[0]);

      $NewCode .= "<tr>".
        (($AOptions['lines'] == "lines")?
         "<td class=\"devcodelines" . ($TrOdd? " devcodelinesodd" : "") . "\" width=\"1%\">" .
          str_pad("".($i + 1), $PadSize, '0', STR_PAD_LEFT) . "</td>":
          ""
        ).
        "<td class=\"devcodelinesarea" . ($TrOdd? " devcodelinesareaodd" : "") . "\"><pre class=\"devcode devcodeline\">" .
        $CodeLines[$i] . "</pre></td></tr>";
      $TrOdd = !$TrOdd;
    }
    $ACode = $NewCode . "</table>";

    $DevFmt_CodeIndex++;
    return '<pre class="devcodeblock" title="'.devfmt_getLangTitle($ALang).'">'.
        '<table class="devcodetools"><tbody><tr>'.
        '<td>&nbsp;'.devfmt_getLangTitle($ALang).'&nbsp;|&nbsp;</td>'.
        '<td style="background-image:url(\''.DEVFMT_URL.'img/devformatter-copy.png\');background-repeat:no-repeat;'.
        'background-position:50% 50%;width:16px;height:16px;"/><embed id="ZeroClipboard'.$DevFmt_CodeIndex.'" src="'.
        DEVFMT_URL.'_zclipboard.swf" loop="false" menu="false" quality="best" bgcolor="#ffffff" width="16px" height="16px"'.
        ' align="middle" allowScriptAccess="always" allowFullScreen="false" type="application/x-shockwave-flash"'.
        ' pluginspage="http://www.macromedia.com/go/getflashplayer" flashvars="id='.$DevFmt_CodeIndex.'&width=16&height=16"'.
        ' wmode="transparent" /></td>'.
        '<td>&nbsp;'.str_replace(" ", "&nbsp;", $DevFmt_Config['copyclipboartext']).'&nbsp;|</td>'.
        '<td style="cursor:pointer" title="DevFormatter Plugin" onclick="devfmt_credits()">?</td>'.
        '<td width="99%">&nbsp;</td></tr></tbody></table>'.
        '<div class="devcodeoverflow"'.$AStyle.'>'.
        $ACode.
        '</div>'.
        '</pre>';
  }

  function devfmt_ContentCheckCode($AContent, $ADevFmt = false){
    GLOBAL $DevFmt_Config;

    $GeshiOperators = Array();
    $GeshiOperators[] = Array("!<code(.*?)>(.*?)</code>!is", 1);
    if($DevFmt_Config['parsepre'])
      $GeshiOperators[] = Array("!<pre lang=(.*?)>(.*?)</pre>!is", 2);

    foreach($GeshiOperators as $GeshiOperatorRule){
      $GeshiOperator = $GeshiOperatorRule[0];
      $codes = array();
      $unid = uniqid('');
      $i = 0;
      while(preg_match($GeshiOperator, $AContent, $matches)){
        $i++;
        $codes[$i]['options'] = $matches[1];
        if($brformat)
          if(trim($matches[2]) != ""){
            $codelines = explode("\r\n", $matches[2]);
            if(trim($codelines[0]) == "")
              array_shift($codelines);

            $matches[2] = implode("\r\n", $codelines);
          }

        $codes[$i]['code'] = $matches[2];
        $AContent = str_replace($matches[0], $unid.$i." ", $AContent);
      }

      $i = 0;
      while(preg_match("!".$unid."([0-9]+) !is", $AContent, $matches)){
        $i++;
        $opts = array();
        if(trim($codes[$i]['options']) != ""){
          if($GeshiOperatorRule[1] == 1){
            $optsdata = explode(" ", $codes[$i]['options']);
            foreach($optsdata as $option){
              $opt = explode("=", $option);
              $opt[1] = trim($opt[1]);
              if($opt[1][0] == "\"")
                $opt[1] = substr($opt[1], 1, strlen($opt[1]) - 2);
              $opts[strtolower($opt[0])] = $opt[1];
            }
          }elseif($GeshiOperatorRule[1] == 2){
            $optsdata = explode(" ", $codes[$i]['options']);
            $opts['lang'] = trim(array_shift($optsdata), " \"");
            foreach($optsdata as $option){
              $opt = explode("=", $option);
              $opt[1] = trim($opt[1]);
              if($opt[1][0] == "\"")
                $opt[1] = substr($opt[1], 1, strlen($opt[1]) - 2);
              $opts[strtolower($opt[0])] = $opt[1];
            }
          }
        }

        if(IsSet($opts['lang'])){
          $opts['lang'] = StrToLower($opts['lang']);
          if(strpos($opts['lang'], '[') !== false){
            if(preg_match("/\[(.*?)\]/", $opts['lang'], $match)){
              $tmpOpts = explode("-", $match[1]);
              $opts['options'] = Array();
              if(is_array($tmpOpts))
                foreach($tmpOpts as $Opt)
                  $opts['options'][$Opt] = $Opt;

              $opts['lang'] = str_replace($match[0], '', $opts['lang']);
            }
          }
          if(!IsSet($opts['options'])){
            $opts['options'] = Array();
          }

          if($DevFmt_Config['displaylinenumbers'])
            $opts['options']['lines'] = "lines";

          if(!IsSet($opts['style'])){
            $opts['style'] = "";
          }

          $code = devfmt_ParseStructure(
            devfmt_ParseCode($codes[$i]['code'], $opts['lang'], $ADevFmt),
            $opts['lang'], $opts['options'], $opts['style']);
        }else{
          $code = "<pre>".$codes[$i]['code']."</pre>";
        }

        $AContent = preg_replace("!".$unid."([0-9]+) !is", $code, $AContent, 1);
      }
    }
    return $AContent;
  }

  function devfmt_ContentFormat($AContent, $AFilter){
    GLOBAL $DefaultFilters, $DevFmt_Config, $DevFmt_SuppLangs, $DevFmt_FiltersToAvoid, $post;

    $AContent = devfmt_ContentCheckCode($AContent, true);

    if(($AFilter == 'example') || ($DevFmt_Config['usedevformat'])){
      if($AFilter == 'comment_text'){
        $AContent = preg_replace('!(<script.*?>)(.*?)</script>!is', '', $AContent);
        $AContent = preg_replace('!(<iframe.*?>)(.*?)</iframe>!is', '', $AContent);
        $AContent = preg_replace('!(<meta.*?)(.*?)>!is', '', $AContent);
      }

      $svScript = devfmt_saveContent($AContent, "<script(.*?)>(.*?)</script>");
      $svStyle = devfmt_saveContent($AContent, "<style(.*?)>(.*?)</style>");
      $svTextarea = devfmt_saveContent($AContent, "<textarea(.*?)>(.*?)</textarea>");
      $svSelect = devfmt_saveContent($AContent, "<select(.*?)>(.*?)</select>");
      $svPre = devfmt_saveContent($AContent, "<pre(.*?)>(.*?)</pre>");

      $AContent = str_replace("\r\n", "\n", $AContent);
      $AContent = str_replace("\n", "\r\n", $AContent);
      $AContent = preg_replace('!(<(pre|blockquote)(.*?)>)([ |\r\n])!', '$1', $AContent);

      $AContent = preg_replace('!(</(p|div|pre|ul|ol|li|dl|dd|dt|table|tr|th|td|h[1-6])>)([ |\r\n])!', '$1', $AContent);
      $AContent = preg_replace('!([ |\r\n])(<(p|div|pre|ul|ol|li|dl|dd|dt|table|tr|th|td)(.*?)>)!', '$2', $AContent);

      $AContent = preg_replace("/\r\n\r\n/", "{{2br}}", $AContent);
      $AContent = preg_replace("/\r\n/", '<br />', $AContent);

      if($DevFmt_Config['highlighttags'] && IsSet($post)){
        $posttags = get_the_tags($post->ID);
        if($posttags){
          $Tags = array();
          foreach($posttags as $tag) $Tags[] = $tag->name;
          arsort($Tags);
          $svAnyTag = devfmt_saveContent($AContent, "<(.*?)>");
          foreach($Tags as $tag)
            $AContent = preg_replace("/([ |\r|>|.|,|\/|:])(".$tag.")([ |\r|<|.|,|\/|:])/is",
              "$1<strong>$2</strong>$3", $AContent);
          devfmt_recoverContent($AContent, $svAnyTag);
          unset($svAnyTag);
        }
      }

      if($AFilter != "example")
        $AContent = devfmt_RunDefaultFilters($AFilter, $AContent, true);

      devfmt_recoverContent($AContent, $svScript);
      unset($svScript);
      devfmt_recoverContent($AContent, $svStyle);
      unset($svStyle);
      devfmt_recoverContent($AContent, $svTextarea);
      unset($svTextarea);
      devfmt_recoverContent($AContent, $svSelect);
      unset($svSelect);
      devfmt_recoverContent($AContent, $svPre);
      unset($svPre);

      $AContent = preg_replace("!({{2br}})!", "<br /><br />", $AContent);

      // Change pre class "code" to "devcode"
      $AContent = preg_replace('/(<pre class=[\'|\"]codeblock[\'|\"](.*?)>)/is', '<pre class="devcodeblock"$2>', $AContent);
      $AContent = preg_replace('/(<span class=[\'|\"]codelines[\'|\"](.*?)>)/is', '<span class="devcodelines"$2>', $AContent);
      $AContent = preg_replace('/(<pre class=[\'|\"]code[\'|\"](.*?)>)/is', '<pre class="devcode"$2>', $AContent);

    }else{
      $AContent = devfmt_RunDefaultFilters($AFilter, $AContent, false);
    }
    return $AContent;
  }

  // BLOG
  function devfmt_Formatter_the_content($AContent){
    GLOBAL $_GET, $DevFmt_Config;
    if(IsSet($_GET['feed']) && (!$DevFmt_Config['hookrss2'])){
      return devfmt_RunDefaultFilters('the_content', $AContent, false);
    }else{
      return devfmt_ContentFormat($AContent, 'the_content');
    }
  }

  function devfmt_Formatter_comment_text($AContent){
    return devfmt_ContentFormat($AContent, 'comment_text');
  }

  function devfmt_Formatter_the_excerpt($AContent){
    return devfmt_ContentFormat($AContent, 'the_excerpt');
  }

  // RSS
  function devfmt_Formatter_the_content_rss($AContent){
    return devfmt_ContentFormat($AContent, 'the_content_rss');
  }

  function devfmt_Formatter_comment_text_rss($AContent){
    return devfmt_ContentFormat($AContent, 'comment_text_rss');
  }

  function devfmt_Formatter_the_excerpt_rss($AContent){
    return devfmt_ContentFormat($AContent, 'the_excerpt_rss');
  }

  // Install DevFormatter Plugin
  devfmt_Install();
?>
